function [out] = gendata( phi, lambda, N, f, g, varargin) 
%GENDATA Generates non-normally distributed random variables conforming to
%a given structural equation model

% Initialize
nLat = size(phi,2);
nMan = 0;
for iii=1:nLat
    nMan=nMan+size(lambda{iii},1);
end
decompLat=chol(phi);
vark=zeros(2,nMan);
cork=zeros(2,nMan);

% Parse optional input
Args=struct('acc', 6, 'acc2', 3, 'xi', eye(nMan), 'samples', 1, ...
    'spec_mom', 0, 'file_name', '', 'folder_name', '','rmse',0);
Args=parseArgs(varargin,Args);


xxx=norminv(10^(-Args.acc):10^(-Args.acc):(1-10^(-Args.acc)),0,1);
hMan=zeros(length(xxx),nMan);
hErr=zeros(length(xxx),nMan);
corrMat=zeros(nMan,nMan);
lambda_Mat=zeros(1,nMan);
i=1;
for iii=1:nLat
    for jjj=1:size(lambda{iii},1)
        lambda_Mat(i)=lambda{iii}(jjj);
        i=i+1;
    end
end

% If desired moments are specified, try to find a suitable function
if (isstruct(Args.spec_mom))
    f_old=f;
    g_old=g;
    
    for iii=1:length(Args.spec_mom.goal)
        if Args.spec_mom.goal~=0
            fg=cell(2,1);
            fg{1}=f_old{iii};
            fg{2}=g_old{iii};
            alpha=get_alpha( Args.spec_mom.goal(iii),...
                fg,lambda_Mat(iii),...
                Args.acc, Args.spec_mom.order_moment(iii));
            
            f{iii}=str2func(strcat('@(xxx)(',num2str(alpha),...
                '.*',strrep(func2str(f_old{iii}), '@(xxx)', ''),'+xxx)'));
            g{iii}=str2func(strcat('@(xxx)(',num2str(alpha),...
                '.*',strrep(func2str(g_old{iii}), '@(xxx)', ''),'+xxx)'));
        end
    end
end

% hz stores the functions that are needed to estimate the distortion 
% and to calculate the manifest variable
hz=cell(2,1);
hz{1}=f; hz{2}=g;

% Calculate true covariance matrix and latent correction matrix
trueCov=zeros(nMan);
i1=1; i2=1;
for iii=1:nLat
    for jjj=1:size(lambda{iii},1)
        for kkk=1:nLat
            for lll=1:size(lambda{kkk},1)
                trueCov(i1,i2)=...
                    lambda{iii}(jjj).*lambda{kkk}(lll).*phi(iii,kkk);
                corrMat(i1,i2)=phi(iii,kkk);
                i2=i2+1;
            end
        end
        i1=i1+1; i2=1;
    end
end
errsigma=diag(sqrt(1-diag(trueCov)));
trueCov=trueCov+errsigma*Args.xi*errsigma;

%Calculate the variance and correlation correction factors
for jjj=1:2
    if (size(hz{jjj},1)==1)
        k=hz{jjj};
        hz{jjj}=cell(nMan,1);
        for iii=1:nMan
            hz{jjj}{iii}=k;
        end       
    end
        for iii=1:size(hz{jjj},1)
            vark(jjj,iii)=1/sqrt(var(feval(hz{jjj}{iii},xxx)));
            cork(jjj,iii)=corr(xxx',feval(hz{jjj}{iii},xxx)'); 
        end
    
end

%Calculate the remaining deviation in the manifest variables and an error 
%covariance matrix correcting the deviation
i=1;
if isstruct(Args.spec_mom)
    man_means=zeros(nMan,1);
end
for jjj=1:nLat
    for iii=1:size(lambda{jjj},1)
        hMan(:,i)=feval(hz{1}{i},xxx)'.*(lambda{jjj}(iii)...
            ./cork(1,i)).*vark(1,i);
        hErr(:,i)=feval(hz{2}{i},xxx).*vark(2,i)...
            .*sqrt(1-(lambda{jjj}(iii)./cork(1,i))^2);
        if isstruct(Args.spec_mom)
            man_means(i)=mean(hMan(:,i)+hErr(:,i));
        end
        i=i+1;
    end
end

steps=round(10^(-Args.acc2)/10^(-Args.acc));
hErr_means=get_y(hErr,steps);
hLat_means=get_y(hMan,steps);
latDevi=zeros(size(trueCov));
for iii=2:nMan
    for jjj=1:(iii-1)
        if corrMat(iii,jjj)==1
            h=cov(hMan(:,iii),hMan(:,jjj));
            latDevi(iii,jjj)=h(1,2);
        else 
            latDevi(iii,jjj)=get_cov(corrMat(iii,jjj),hLat_means(:,iii),...
            hLat_means(:,jjj),xxx([steps:steps:end end]));
        end
    end
end

errCovDiff=trueCov-(latDevi+latDevi');

errCorr=zeros(size(errCovDiff));
for iii=2:nMan
    for jjj=1:(iii-1)
        if (abs(errCovDiff(iii,jjj))>(10^(-6)))
            if (mean(hz{2}{iii}(-100:0.1:100)==-100:0.1:100)<1||...
                    mean(hz{2}{jjj}(-100:0.1:100)==-100:0.1:100)<1)
                errCorr(iii,jjj)=get_r(errCovDiff(iii,jjj),...
                    hErr_means(:,iii),hErr_means(:,jjj),...
                    xxx([steps:steps:end end]));
            else
                errCorr(iii,jjj)=errCovDiff(iii,jjj)/...
                    (sqrt(1-(lambda_Mat(iii)./cork(1,iii))^2)*...
                    sqrt(1-(lambda_Mat(jjj)./cork(1,jjj))^2));
            end
        end
    end
end
errCorr=errCorr+errCorr'+eye(nMan);
decompErr=chol(errCorr);

if(Args.rmse)
    rmse=zeros(Args.samples,length(N));
end

%Calculate manifest variables
for mmm=1:length(N)
    
    man=zeros(N(mmm),nMan);
    if isstruct(Args.spec_mom)
        moments=zeros(Args.samples,nMan);
    end
    if ((~isempty(Args.folder_name))||length(N)>1)
        if (~isempty(Args.folder_name))
            Args.folder_name=strcat(Args.folder_name,'\');
        end
        mkdir(strcat(Args.folder_name,'N_',num2str(N(mmm))));
    end

    for kkk=1:Args.samples
        lat=randn(N(mmm),nLat)*decompLat; %Create latent variables
        err=randn(N(mmm),nMan)*decompErr; %Create errors
        i=1;
        for jjj=1:nLat
            for iii=1:size(lambda{jjj},1)
                man(:,i)=...
                    feval(hz{1}{i},lat(:,jjj)).*(lambda{jjj}(iii)...
                    ./cork(1,i)).*vark(1,i)+...
                    feval(hz{2}{i},err(:,i)).*sqrt(1-(lambda{jjj}(iii)...
                    ./cork(1,i))^2).*vark(2,i);
                i=i+1;
            end
        end
        if (Args.rmse)
            rmse(kkk,mmm)=sqrt(sum(sum((corr(man)-trueCov).^2))/...
                (nMan*nMan-nMan));
        end
        %print samples
        dlmwrite(strcat(Args.file_name,num2str(kkk),'.dat'), ...
            round(man*10^5)/(10^5), 'delimiter', '\t');
        if ((~isempty(Args.folder_name))||length(N)>1)
            movefile(strcat(Args.file_name,num2str(kkk),'.dat'),...
                strcat(Args.folder_name,'N_',num2str(N(mmm))))
        end
        if isstruct(Args.spec_mom)
            for iii=1:nMan
                moments(kkk,iii)=mean((man(:,iii)-man_means(iii))...
                    .^Args.spec_mom.order_moment(iii));
            end
        end
    end
    %print rmse
    if (Args.rmse)
        dlmwrite('rmse.dat', round(rmse*10^7)/(10^7), 'delimiter', '\t');
        if (~isempty(Args.folder_name))
            movefile('rmse.dat',Args.folder_name)
        end
    end

    %print moment
    if isstruct(Args.spec_mom)
        dlmwrite('moments.dat', round(moments*10^5)/(10^5),...
            'delimiter', '\t');
        if ((~isempty(Args.folder_name))||length(N)>1)
            movefile('moments.dat',strcat(Args.folder_name,...
                'N_',num2str(N(mmm))))
        end
        out.moments=moments;
    end
end

if (Args.rmse)
    out.rmse=rmse;
end

out.trueCov=trueCov;
out.errCorr=errCorr;
out.lat=lat;
out.err=err;
out.vark=vark;
out.cork=cork;
out.man=man;
end
